@@ -1194,7 +1194,7 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
 
 static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_packet *op) {
   xine_flac_metadata_header header;
-  xine_flac_streaminfo_block streaminfo;
+  xine_flac_streaminfo_block streaminfo = {};
   buf_element_t *buf;
   xine_waveformatex wave;
 
@@ -2166,6 +2166,9 @@ static const char *ogg_get_extensions (demux_class_t *this_gen) {
 static const char *ogg_get_mimetypes (demux_class_t *this_gen) {
   return "application/ogg: ogx: Ogg Stream;"
          "application/x-ogg: ogx: Ogg Stream;"
+         "application/x-ogm: ogx: Ogg Stream;"
+         "application/x-ogm-audio: oga: Ogg Audio;"
+         "application/x-ogm-video: ogv: Ogg Video;"
          "audio/ogg: oga: Ogg Audio;"
          "audio/x-ogg: oga: Ogg Audio;"
          "video/ogg: ogv: Ogg Video;"
