@@ -15,13 +15,11 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Read from a tcp network stream over a lan (put a tweaked mp1e encoder the
  * other end and you can watch tv anywhere in the house ..)
  *
- * $Id: input_net.c,v 1.70 2007/01/19 01:05:25 dgp85 Exp $
- *
  * how to set up mp1e for use with this plugin:
  * 
  * use mp1 to capture the live stream, e.g.
@@ -413,6 +411,8 @@ static int net_plugin_open (input_plugin_t *this_gen ) {
   char *filename;
   char *pptr;
   int port = 7658;
+  int toread = MAX_PREVIEW_SIZE;
+  int trycount = 0;
 
   filename = this->host_port;
   pptr=strrchr(filename, ':');
@@ -431,11 +431,15 @@ static int net_plugin_open (input_plugin_t *this_gen ) {
   /*
    * fill preview buffer
    */
+  while ((toread > 0) && (trycount < 10)) {
 #ifndef WIN32
-  this->preview_size = read (this->fh, this->preview, MAX_PREVIEW_SIZE);
+    this->preview_size += read (this->fh, this->preview + this->preview_size, toread);
 #else
-  this->preview_size = recv (this->fh, this->preview, MAX_PREVIEW_SIZE, 0);
+    this->preview_size += recv (this->fh, this->preview + this->preview_size, toread, 0);
 #endif
+    trycount++;
+    toread = MAX_PREVIEW_SIZE - this->preview_size;
+  }
 
   this->curpos       = 0;
 
