@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: rtsp.c,v 1.18.2.1 2005/04/20 17:21:09 mroi Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * a minimalistic implementation of rtsp protocol,
  * *not* RFC 2326 compilant yet.
@@ -417,6 +415,7 @@ int rtsp_read_data(rtsp_t *s, char *buffer, unsigned int size) {
       rest = malloc(sizeof(char)*17);
       sprintf(rest,"CSeq: %u", seq);
       rtsp_put(s, rest);
+      free(rest);
       rtsp_put(s, "");
       i=_x_io_tcp_read(s->stream, s->s, buffer, size);
     } else
@@ -441,7 +440,8 @@ rtsp_t *rtsp_connect(xine_stream_t *stream, const char *mrl, const char *user_ag
   rtsp_t *s = malloc(sizeof(rtsp_t));
   char *mrl_ptr=strdup(mrl);
   char *slash, *colon;
-  int hostend, pathbegin, i;
+  int hostend, i;
+  size_t pathbegin;
   
   if (strncmp(mrl,"rtsp://",7))
   {
@@ -561,6 +561,7 @@ char *rtsp_search_answers(rtsp_t *s, const char *tag) {
   while (*answer) {
     if (!strncasecmp(*answer,tag,strlen(tag))) {
       ptr=strchr(*answer,':');
+      if (!ptr) return NULL;
       ptr++;
       while(*ptr==' ') ptr++;
       return ptr;
